/*
 * Copyright (c) 2021 The Hydra authors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package be.ugent.zeus.hydra.feed.operations;

import java.util.function.IntPredicate;
import java.util.function.Supplier;

import be.ugent.zeus.hydra.feed.HomeFeedRequest;
import be.ugent.zeus.hydra.feed.cards.Card;

/**
 * Factory to get operations on a feed.
 *
 * @author Niko Strijbol
 */
public final class OperationFactory {

    /**
     * Create an operation that will insert the request to the feed.
     *
     * @param request The request to insert.
     * @return The operation.
     */
    public static FeedOperation add(HomeFeedRequest request) {
        return new RequestOperation(request);
    }

    /**
     * Create an operation that will remove a card type from the feed.
     *
     * @param cardType The type of card to remove.
     * @return The operation.
     */
    @SuppressWarnings("WeakerAccess")
    public static FeedOperation del(@Card.Type int cardType) {
        return new RemoveOperation(cardType);
    }

    /**
     * Utility method to get an operation. If {@code isActive} evaluates to true, this method will call
     * {@link #add(HomeFeedRequest)}, and otherwise {@link #del(int)}.
     *
     * @param isIgnored If the card type should be ignored or not. Must return true if the card must be ignored.
     * @param supplier  The request supplier. Use a supplier so the request only gets made when needed.
     * @param type      Type of the home card.
     * @return The operation.
     */
    public static FeedOperation get(IntPredicate isIgnored, Supplier<HomeFeedRequest> supplier, @Card.Type int type) {
        if (isIgnored.test(type)) {
            return del(type);
        } else {
            return add(supplier.get());
        }
    }
}
